<?php
/**
 * A Menu template engine class
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class MenuTemplateEngine extends BaseTemplateEngine implements CompileableTemplate {
	/**
	 * Main nodes in the XML tree ('menu' is ignored)
	 */
	private $mainNodes = array(
		'block-list',
	);

	/**
	 * Sub nodes in the XML tree
	 */
	private $subNodes = array(
		'entry-list',
		'entry',
		'entry-id',
		'entries-content',
		'header',
		'footer',
		'footer-id',
		'footer-class',
		'footer-text',
		'block',
		'title',
		'title-id',
		'title-class',
		'title-text',
		'design',
		'text',
		'advert',
		'anchor',
		'anchor-id',
		'anchor-text',
		'anchor-title',
		'anchor-href',
	);

	/**
	 * Menu instance
	 */
	private $menuInstance = null;

	/**
	 * Current main node
	 */
	private $curr = array();

	/**
	 * Content from depency
	 */
	private $depencyContent = array();

	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of the class TemplateEngine and prepares it for usage
	 *
	 * @param	$appInstance	A manageable application
	 * @param	$menuInstance	A RenderableMenu instance
	 * @return	$tplInstance	An instance of TemplateEngine
	 * @throws	BasePathIsEmptyException		If the provided $templateBasePath is empty
	 * @throws	InvalidBasePathStringException	If $templateBasePath is no string
	 * @throws	BasePathIsNoDirectoryException	If $templateBasePath is no
	 *											directory or not found
	 * @throws	BasePathReadProtectedException	If $templateBasePath is
	 *											read-protected
	 */
	public final static function createMenuTemplateEngine (ManageableApplication $appInstance, RenderableMenu $menuInstance) {
		// Get a new instance
		$tplInstance = new MenuTemplateEngine();

		// Get language and file I/O instances from application
		$langInstance = $appInstance->getLanguageInstance();
		$ioInstance = $appInstance->getFileIoInstance();

		// Determine base path
		$templateBasePath = $tplInstance->getConfigInstance()->getConfigEntry('application_base_path') . $appInstance->getRequestInstance()->getRequestElement('app') . '/';

		// Is the base path valid?
		if (empty($templateBasePath)) {
			// Base path is empty
			throw new BasePathIsEmptyException($tplInstance, self::EXCEPTION_UNEXPECTED_EMPTY_STRING);
		} elseif (!is_string($templateBasePath)) {
			// Is not a string
			throw new InvalidBasePathStringException(array($tplInstance, $templateBasePath), self::EXCEPTION_INVALID_STRING);
		} elseif (!is_dir($templateBasePath)) {
			// Is not a path
			throw new BasePathIsNoDirectoryException(array($tplInstance, $templateBasePath), self::EXCEPTION_INVALID_PATH_NAME);
		} elseif (!is_readable($templateBasePath)) {
			// Is not readable
			throw new BasePathReadProtectedException(array($tplInstance, $templateBasePath), self::EXCEPTION_READ_PROTECED_PATH);
		}

		// Get configuration instance
		$configInstance = FrameworkConfiguration::getInstance();

		// Set the base path
		$tplInstance->setTemplateBasePath($templateBasePath);

		// Set the language and IO instances
		$tplInstance->setLanguageInstance($langInstance);
		$tplInstance->setFileIoInstance($ioInstance);

		// Set template extensions
		$tplInstance->setRawTemplateExtension($configInstance->getConfigEntry('raw_template_extension'));
		$tplInstance->setCodeTemplateExtension($configInstance->getConfigEntry('menu_template_extension'));

		// Absolute output path for compiled templates
		$tplInstance->setCompileOutputPath($configInstance->getConfigEntry('base_path') . $configInstance->getConfigEntry('compile_output_path'));

		// Set the menu instance
		$tplInstance->setMenuInstance($menuInstance);

		// Return the prepared instance
		return $tplInstance;
	}

	/**
	 * Load a specified menu template into the engine
	 *
	 * @param	$template	The menu template we shall load which is
	 *						located in 'menu' by default
	 * @return	void
	 */
	public function loadMenuTemplate ($template) {
		// Set template type
		$this->setTemplateType($this->getConfigInstance()->getConfigEntry('menu_template_type'));

		// Load the special template
		$this->loadTemplate($template);
	}

	/**
	 * Getter for current main node
	 *
	 * @return	$currMainNode	Current main node
	 */
	public final function getCurrMainNode () {
		return $this->curr['main_node'];
	}

	/**
	 * Setter for current main node
	 *
	 * @param	$element		Element name to set as current main node
	 * @return	$currMainNode	Current main node
	 */
	private final function setCurrMainNode ($element) {
		$this->curr['main_node'] = (string) $element;
	}

	/**
	 * Getter for main node array
	 *
	 * @return	$mainNodes	Array with valid main node names
	 */
	public final function getMainNodes () {
		return $this->mainNodes;
	}

	/**
	 * Getter for sub node array
	 *
	 * @return	$subNodes	Array with valid sub node names
	 */
	public final function getSubNodes () {
		return $this->subNodes;
	}

	/**
	 * Handles the start element of an XML resource
	 *
	 * @param	$resource		XML parser resource (currently ignored)
	 * @param	$element		The element we shall handle
	 * @param	$attributes		All attributes
	 * @return	void
	 * @throws	InvalidXmlNodeException		If an unknown/invalid XML node name was found
	 */
	public function startElement ($resource, $element, array $attributes) {
		// Initial method name which will never be called...
		$methodName = 'initMenu';

		// Make the element name lower-case
		$element = strtolower($element);

		// Is the element a main node?
		//* DEBUG: */ echo "START: &gt;".$element."&lt;<br />\n";
		if (in_array($element, $this->getMainNodes())) {
			// Okay, main node found!
			$methodName = 'start' . $this->convertToClassName($element);

			// Set it
			$this->setCurrMainNode($element);
		} elseif (in_array($element, $this->getSubNodes())) {
			// Sub node found
			$methodName = 'start' . $this->convertToClassName($element);
		} elseif ($element != 'menu') {
			// Invalid node name found
			throw new InvalidXmlNodeException(array($this, $element, $attributes), XmlParser::EXCEPTION_XML_NODE_UNKNOWN);
		}

		// Call method
		//* DEBUG: */ echo "call: ".$methodName."<br />\n";
		call_user_func_array(array($this, $methodName), $attributes);
	}

	/**
	 * Ends the main or sub node by sending out the gathered data
	 *
	 * @param	$resource	An XML resource pointer (currently ignored)
	 * @param	$nodeName	Name of the node we want to finish
	 * @return	void
	 * @throws	XmlNodeMismatchException	If current main node mismatches the closing one
	 */
	public function endElement ($resource, $nodeName) {
		// Make all lower-case
		$nodeName = strtolower($nodeName);

		// Does this match with current main node?
		//* DEBUG: */ echo "END: &gt;".$nodeName."&lt;<br />\n";
		if (($nodeName != $this->getCurrMainNode()) && (in_array($nodeName, $this->getMainNodes()))) {
			// Did not match!
			throw new XmlNodeMismatchException (array($this, $nodeName, $this->getCurrMainNode()), XmlParser::EXCEPTION_XML_NODE_MISMATCH);
		} // END - if

		// Construct method name
		$methodName = 'finish' . $this->convertToClassName($nodeName);

		// Call the corresponding method
		//* DEBUG: */ echo "call: ".$methodName."<br />\n";
		call_user_func_array(array($this, $methodName), array());
	}

	/**
	 * Currently not used
	 *
	 * @param	$resource		XML parser resource (currently ignored)
	 * @param	$characters		Characters to handle
	 * @return	void
	 * @todo	Find something useful with this!
	 */
	public function characterHandler ($resource, $characters) {
		// Trim all spaces away
		$characters = trim($characters);

		// Is this string empty?
		if (empty($characters)) {
			// Then skip it silently
			return false;
		} // END - if

		// Unfinished work!
		$this->partialStub('Handling extra characters is not yet supported! length='.strlen($characters));
	}

	/**
	 * Handles the template depency for given node
	 *
	 * @param	$node	The node we should load a depency template
	 * @param	$templateDepency	A template to load to satisfy depencies
	 * @return	void
	 */
	private function handleTemplateDepency ($node, $templateDepency) {
		// Is the template depency set?
		if ((!empty($templateDepency)) && (!isset($this->depencyContent[$node]))) {
			// Get a temporay menu template instance
			$templateInstance = ObjectFactory::createObjectByConfiguredName('menu_template_class', array($this->getApplicationInstance(), $this->getMenuInstance()));

			// Then load it
			$templateInstance->loadMenuTemplate($templateDepency);

			// Get an XmlParser instance
			$templateInstance->renderXmlContent();

			// Parse the template's content contents
			$this->depencyContent[$node] = $templateInstance->getRawTemplateData();
		} // END - if
	}

	/**
	 * Intializes the menu
	 *
	 * @param	$templateDepency	A template to load to satisfy depencies
	 * @return	void
	 * @todo	Add cache creation here
	 */
	private function initMenu ($templateDepency = '') {
		// Get web template engine
		$this->setTemplateInstance(ObjectFactory::createObjectByConfiguredName('web_template_class', array($this->getApplicationInstance())));

		// Handle the depency template
		$this->handleTemplateDepency('menu', $templateDepency);

		// Load the header template for this page
		$this->getTemplateInstance()->loadCodeTemplate('menu_global_start');

		// Set the variable group to page
		$this->setVariableGroup('menu');

		// Set its content in this template instance
		$this->assignVariable('menu_start', $this->getTemplateInstance()->getRawTemplateData());
	}

	/**
	 * Finishes the menu
	 *
	 * @return	void
	 */
	private function finishMenu () {
		// Load the header template for this page
		$this->getTemplateInstance()->loadCodeTemplate('menu_global_end');

		// Set the variable group to page
		$this->setVariableGroup('menu');

		// Set its content in this template instance
		$this->assignVariable('menu_end', $this->getTemplateInstance()->getRawTemplateData());
	}

	/**
	 * Starts the menu entries by loading a (maybe) provided template depency
	 *
	 * @param	$templateDepency	A template to load to satisfy depencies
	 * @return	void
	 */
	private function startEntryList ($templateDepency = '') {
		// Handle the depency template
		$this->handleTemplateDepency('entries', $templateDepency);

		// Load the header template for this page
		$this->getTemplateInstance()->loadCodeTemplate('menu_entries_start');

		// Set the variable group to page
		$this->setVariableGroup('menu');

		// Set its content in this template instance
		$this->assignVariable('entries_start', $this->getTemplateInstance()->getRawTemplateData());
	}

	/**
	 * Finishes the menu entries
	 *
	 * @return	void
	 */
	private function finishEntryList () {
		// Load the header template for this page
		$this->getTemplateInstance()->loadCodeTemplate('menu_entries_end');

		// Set the variable group to page
		$this->setVariableGroup('menu');

		// Set its content in this template instance
		$this->assignVariable('entries_end', $this->getTemplateInstance()->getRawTemplateData());
	}

	/**
	 * Starts the menu header
	 *
	 * @return	void
	 */
	private function startHeader () {
		// Do we have a template instance?
		if (is_null($this->getTemplateInstance())) {
			// Init template instance for underlaying web templates
			$templateInstance = ObjectFactory::createObjectByConfiguredName('web_template_class');

			// Set it in this template engine
			$this->setTemplateInstance($templateInstance);
		} // END - if

		// Load the header template for this page
		$this->getTemplateInstance()->loadCodeTemplate('menu_header_start');

		// Set the variable group to page
		$this->setVariableGroup('menu');

		// Set its content in this template instance
		$this->assignVariable('header', $this->getTemplateInstance()->getRawTemplateData());
	}

	/**
	 * Finishes the menu header
	 *
	 * @return	void
	 */
	private function finishHeader () {
		// Load the header template for this page
		$this->getTemplateInstance()->loadCodeTemplate('menu_header_end');

		// Set the variable group to page
		$this->setVariableGroup('menu');

		// Set its content in this template instance
		$this->assignVariable('header_end', $this->getTemplateInstance()->getRawTemplateData());
	}

	/**
	 * Starts the menu footer
	 *
	 * @return	void
	 */
	private function startFooter () {
		// Do we have a template instance?
		if (is_null($this->getTemplateInstance())) {
			// Init template instance for underlaying web templates
			$templateInstance = ObjectFactory::createObjectByConfiguredName('web_template_class');

			// Set it in this template engine
			$this->setTemplateInstance($templateInstance);
		} // END - if

		// Load the footer template for this page
		$this->getTemplateInstance()->loadCodeTemplate('menu_footer_start');

		// Set the variable group to page
		$this->setVariableGroup('menu');

		// Set its content in this template instance
		$this->assignVariable('footer', $this->getTemplateInstance()->getRawTemplateData());
	}

	/**
	 * Finishes the menu footer
	 *
	 * @return	void
	 */
	private function finishFooter () {
		// Load the footer template for this page
		$this->getTemplateInstance()->loadCodeTemplate('menu_footer_end');

		// Set the variable group to page
		$this->setVariableGroup('menu');

		// Set its content in this template instance
		$this->assignVariable('footer_end', $this->getTemplateInstance()->getRawTemplateData());
	}

	/**
	 * Starts the menu property 'title'
	 *
	 * @return	void
	 */
	private function startTitle () {
		$this->partialStub('Cleared due to XML rewrite.');
	}

	/**
	 * Finishes the title node by added another template to the menu
	 *
	 * @return	void
	 */
	private function finishTitle () {
		$this->partialStub('Cleared due to XML rewrite.');
	}

	/**
	 * Starts the menu text
	 *
	 * @return	void
	 */
	private function startText () {
		// Do we have a template instance?
		if (is_null($this->getTemplateInstance())) {
			// Init template instance for underlaying web templates
			$templateInstance = ObjectFactory::createObjectByConfiguredName('web_template_class');

			// Set it in this template engine
			$this->setTemplateInstance($templateInstance);
		} // END - if

		// Load the text template for this page
		$this->getTemplateInstance()->loadCodeTemplate('menu_text_start');

		// Set the variable group to page
		$this->setVariableGroup('menu');

		// Set its content in this template instance
		$this->assignVariable('text', $this->getTemplateInstance()->getRawTemplateData());
	}

	/**
	 * Finishes the menu text
	 *
	 * @return	void
	 */
	private function finishText () {
		// Load the text template for this page
		$this->getTemplateInstance()->loadCodeTemplate('menu_text_end');

		// Set the variable group to page
		$this->setVariableGroup('menu');

		// Set its content in this template instance
		$this->assignVariable('text_end', $this->getTemplateInstance()->getRawTemplateData());
	}

	/**
	 * Starts the menu property 'entry'
	 *
	 * @return	void
	 */
	private function startEntry () {
		$this->partialStub('Cleared due to XML rewrite.');
	}

	/**
	 * Finishes the entry node by added another template to the menu
	 *
	 * @return	void
	 */
	private function finishEntry () {
		$this->partialStub('Cleared due to XML rewrite.');
	}

	/**
	 * Starts the menu property 'anchor'
	 *
	 * @param	$id		Id of the anchor
	 * @param	$link	Link text of the anchor
	 * @param	$title	Link title of the anchor
	 * @return	void
	 */
	private function startAnchor () {
		$this->partialStub('Please implement this method.');
	}

	/**
	 * Finishes the anchor node by added another template to the menu
	 *
	 * @return	void
	 */
	private function finishAnchor () {
		$this->partialStub('Please implement this method.');
	}

	/**
	 * Getter for menu cache file (FQFN)
	 *
	 * @return	$fqfn	Full-qualified file name of the menu cache
	 */
	public function getMenuCacheFqfn () {
		// Get the FQFN ready
		$fqfn = sprintf("%s%s%s/%s.%s",
			$this->getConfigInstance()->getConfigEntry('base_path'),
			$this->getGenericBasePath(),
			'menus/_cache',
			md5(
				$this->getMenuInstance()->getMenuName() . ':' .
				$this->__toString() . ':' .
				$this->getMenuInstance()->__toString()
			),
			$this->getMenuInstance()->getMenuType()
		);

		// Return it
		return $fqfn;
	}
}

// [EOF]
?>
